__version__ = "5.1.8"

from .file_storage import FileStorage, ResourceId
from .models.base import Base
from .models.base_config_models import GigaChatConfig
from .models.chat import Chat, Context, ChatMessage, AIMessage, HumanMessage, MiscMessage, make_content
from .models.chat_item import ChatItem, OuterContextItem, InnerContextItem, ReplicaItem
from .models.enums import MTRSLabelEnum, DiagnosticsXMLTagEnum, MTRSXMLTagEnum, DoctorChoiceXMLTagEnum
from .models.tracks import TrackInfo, DomainInfo
from .models.widget import Widget
from .parallel_map import parallel_map
from .utils import make_session_id, read_json, try_parse_json, try_parse_int, try_parse_float, pretty_line
from .validators import ExistingPath, ExistingFile, ExistingDir, StrNotEmpty, SecretStrNotEmpty, Prompt, Message
from .xml_parser import XMLParser



from pydantic import Field

from .base import Base


class TrackInfo(Base):
    track_id: str = Field(alias="TrackId")
    name: str = Field(alias="Name")
    domain_id: str = Field(alias="DomainId")


class DomainInfo(Base):
    domain_id: str = Field(alias="DomainId")
    name: str = Field(alias="Name")



from typing import Any, Callable, TypeVar, Iterable
import concurrent.futures
from tqdm import tqdm


X = TypeVar("X")


def parallel_map(
    func: Callable[..., X],
    items: Iterable[Any],
    process: bool = False,
    multiple_args: bool = False,
    max_workers: int = 2,
    show_tqdm: bool = False,
    desc: str = "",
) -> list[X]:
    pool = (
        concurrent.futures.ProcessPoolExecutor(max_workers=max_workers)
        if process
        else concurrent.futures.ThreadPoolExecutor(max_workers=max_workers)
    )
    with pool as executor:
        futures = []
        for item in items:
            if multiple_args:
                futures.append(executor.submit(func, *item))
            else:
                futures.append(executor.submit(func, item))
        results: list[X] = [future.result() for future in tqdm(futures, disable=(not show_tqdm), desc=desc)]
    return results



from typing import List, Self, Literal

from pydantic import BaseModel, model_validator


class Widget(BaseModel):
    type: Literal["widget"] = "widget"
    buttons: List[List[str]] | None = None
    ibuttons: List[List[str]] | None = None

    @model_validator(mode="after")
    def check(self) -> Self:
        if not self.buttons and not self.ibuttons:
            raise ValueError("Empty widget is not allowed!")
        if not self.ibuttons:
            return self
        for row in self.ibuttons:
            for btn in row:
                if ":" in btn:
                    continue
                raise ValueError(f"Expected buttons like `<callback>:<caption>`, found: {btn}")
        return self
